require "defines"

local floor = math.floor
local min = math.min

-- Constants
local FRAME_NAME = "dark_game_clock"
local VALID_LOCATIONS = {
  left = true,
  top = true,
  center = true
 }
local UPDATE_INTERVAL = 6 -- minimal update interval
local SETTINGS = {
  enabled = "boolean",
  show_clock = "boolean",
  show_run_clock = "boolean",
  style = "string",
  minute_rounding = "number",
  location = "string",
  show_minutes = "boolean",
  show_seconds_in_runtime = "boolean",
  frame_leftpadding = "number"
}

-- globals
local frame = nil

-- gui stuff
local function remove_gui()
  frame = nil
  for loc, _ in pairs(VALID_LOCATIONS) do
    if game.player.gui[loc][FRAME_NAME] then game.player.gui[loc][FRAME_NAME].destroy() end
  end
end

local function init_gui()
  remove_gui()

  if not glob.enabled then return end

  frame = game.player.gui[glob.location].add{type="frame", name=FRAME_NAME, caption = '', style='naked_frame_style', direction='vertical'}
  frame.style.leftpadding = glob.frame_leftpadding
  
  if glob.show_clock then
    frame.add{type="label", name="game_clock", caption=""}
    local style_game = frame.game_clock.style
    style_game.font = "default-frame"
  end
  
  if glob.show_run_clock then
    frame.add{type="label", name="run_clock", caption=""}
  end

end

local function do_update()
  if not frame then return end
  
  if glob.show_clock and frame.game_clock then
    local time_normal = game.daytime*24
    -- flip time around, 0.5 is midnight ugh
    if time_normal > 12 then time_normal = time_normal - 12 else time_normal = time_normal + 12 end

    local ampm = ""
    local minute = ""
    local hour = floor(time_normal)

    if glob.show_minutes then
      local raw_mins = floor((time_normal % 1)*60)
      
      minute = string.format(":%02d", raw_mins - raw_mins % glob.minute_rounding )
    end

    if glob.style == "12h" then 
      ampm = (hour >= 12 and " PM") or " AM"
      hour = (hour == 0 and 12) or (hour > 12 and (hour - 12)) or hour
    end

    frame.game_clock.caption = string.format("%02d%s%s", hour, minute, ampm)
  end
  
  if glob.show_run_clock and frame.run_clock then
    local run_time_s = floor(game.tick/60)
    local run_time_minutes = floor(run_time_s/60)
    local run_time_hours = floor(run_time_minutes/60)
    local run_time_days = floor(run_time_hours/24)
    local days = ""
    if run_time_days > 0 then days = string.format("%d days, ", run_time_days) end
    local seconds = ""
    if glob.show_seconds_in_runtime then
      seconds = string.format(":%02d", run_time_s % 60)
    end
    frame.run_clock.caption = string.format("%s%02d:%02d%s", days, run_time_hours % 24, run_time_minutes % 60, seconds)
  end

end 

local function set_defaults()
  -- set defaults
  glob.frame_leftpadding = glob.frame_leftpadding or 20
  
  glob.location = glob.location or "left"
  glob.style = glob.style or "24h"
  glob.minute_rounding = glob.minute_rounding or 15
  glob.enabled = (type(glob.enabled) == "nil" ) and true or glob.enabled
  
  glob.show_minutes = (type(glob.show_minutes) == "nil" ) and true or glob.show_minutes
  
  glob.show_seconds_in_runtime = (type(glob.show_seconds_in_runtime) == "nil" ) and false or glob.show_seconds_in_runtime
  
  glob.show_clock = (type(glob.show_clock) == "nil" ) and true or glob.show_clock
  glob.show_run_clock = (type(glob.show_run_clock) == "nil" ) and true or glob.show_run_clock
  
end

-- events
game.oninit(function()
end)

game.onevent(defines.events.onplayercreated, function(event)
  set_defaults()
  init_gui()
  do_update()
end)

game.onload(
function()
  -- restore gui state on load or rigth after save
  set_defaults()
  init_gui()
  do_update()
end)

game.onsave(function()
  -- alow mod to be removed along with gui at any time
  remove_gui()
end)

game.onevent(defines.events.ontick, function(event)

  --if glob.force_reinit then init_gui(); glob.force_reinit = false; end
  if (game.tick % min(UPDATE_INTERVAL*glob.minute_rounding, 60) == 0) then 
    do_update()
  end 
  
end)


remote.addinterface("TimeDisplay", {
  -- remote.call("TimeDisplay", "SetSetting", "setting_name", value)
  --[[
  remote.call("TimeDisplay", "SetSetting", "enabled", true/false)         -- controls global mod state                       [default: true]
  remote.call("TimeDisplay", "SetSetting", "show_clock", true/false)      -- controls display of day time clock              [default: true]
  remote.call("TimeDisplay", "SetSetting", "show_run_clock", true/false)  -- controls display of run time clock              [default: true]
  remote.call("TimeDisplay", "SetSetting", "style", "12h/24h")            -- style for day time clock                        [default: 24h]
  remote.call("TimeDisplay", "SetSetting", "location", "left/top/center") -- display location                               [default: left]
  remote.call("TimeDisplay", "SetSetting", "minute_rounding", 1-60)       -- rounds minutes in daytime clock to this value   [default: 15]
  remote.call("TimeDisplay", "SetSetting", "show_minutes", true/false)    -- show/hide minutes in day time clock             [default: true]
  remote.call("TimeDisplay", "SetSetting", "show_seconds_in_runtime", true/false) -- should run time clock show seconds      [default: false]
  remote.call("TimeDisplay", "SetSetting", "frame_leftpadding", 0-9999 )  -- shifts display horizontally, set to 1450 on 1080p monitor to move display near minimap [default: 20]
  ]]--
  SetSetting = function(setting, value)
    if not SETTINGS[setting] then
      game.player.print("Unknown setting "..setting )
      return
    end
    -- check type
    if type(value) ~= SETTINGS[setting] then
      game.player.print("Value type missmatch, expected "..SETTINGS[setting].." got "..type(value))
      return
    end
    -- check values
    if setting == "style" and (value ~= "12h" and value ~= "24h") then
      game.player.print("Unexpected value, expected 12h or 24h")
      return
    end    
    if setting == "location" and not VALID_LOCATIONS[value] then
      game.player.print("Unexpected value, expected left, top or center")
      return
    end    
    
    if setting == "minute_rounding" and (value > 60 or value < 1) then
      game.player.print("Unexpected value, expected 1-60")
      return
    end
    
    -- appply
    glob[setting] = value
    init_gui()
    do_update()
  end
})

